var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  dataProvider: () => dataProvider,
  defaultHandler: () => defaultHandler,
  deleteHandler: () => deleteHandler,
  getListHandler: () => getListHandler,
  getManyHandler: () => getManyHandler,
  getOneHandler: () => getOneHandler,
  updateHandler: () => updateHandler
});
module.exports = __toCommonJS(src_exports);

// src/getListHandler.ts
var getListHandler = async (req, res, table, arg, options) => {
  var _a, _b, _c, _d, _e, _f;
  const { pagination, sort, filter } = req.body.params;
  let queryArgs = {
    findManyArg: {
      select: (_a = arg == null ? void 0 : arg.select) != null ? _a : void 0,
      include: (_b = arg == null ? void 0 : arg.include) != null ? _b : void 0,
      where: (_c = arg == null ? void 0 : arg.where) != null ? _c : {}
    },
    countArg: {
      where: (_d = arg == null ? void 0 : arg.where) != null ? _d : {}
    }
  };
  if (filter) {
    Object.entries(filter).forEach(([colName, value]) => {
      if (colName.startsWith("_"))
        return;
      if (colName === "id" || colName === "uuid" || colName === "cuid" || colName.endsWith("_id") || typeof value === "number" || typeof value === "boolean") {
        queryArgs.findManyArg.where[colName] = value;
        queryArgs.countArg.where[colName] = value;
      } else if (Array.isArray(value)) {
        queryArgs.findManyArg.where[colName] = { in: value };
        queryArgs.countArg.where[colName] = { in: value };
      } else if (typeof value === "string") {
        queryArgs.findManyArg.where[colName] = { contains: value };
        queryArgs.countArg.where[colName] = { contains: value };
      }
    });
  }
  if (pagination) {
    const { page, perPage } = pagination;
    const first = (page - 1) * perPage;
    const last = page * perPage - 1;
    if (first === 0 && last === 999) {
    } else {
      queryArgs.findManyArg.skip = first;
      queryArgs.findManyArg.take = last - first + 1;
    }
  }
  if (sort) {
    const { field, order } = sort;
    if (field && order) {
      if (field.includes(".")) {
        const [relation, subfield] = field.split(".");
        queryArgs.findManyArg.orderBy = {
          [relation]: { [subfield]: order.toLowerCase() }
        };
      } else {
        queryArgs.findManyArg.orderBy = { [field]: order.toLowerCase() };
      }
    }
    if ((_e = options == null ? void 0 : options.noNullsOnSort) == null ? void 0 : _e.includes(field)) {
      queryArgs.findManyArg.where = { [field]: { not: null } };
      queryArgs.countArg.where = { [field]: { not: null } };
    }
  }
  if (options == null ? void 0 : options.debug) {
    console.log("queryArgs", JSON.stringify(queryArgs, null, 2));
  }
  const [data, total] = await Promise.all([
    table.findMany(queryArgs.findManyArg),
    table.count(queryArgs.countArg)
  ]);
  await ((_f = options == null ? void 0 : options.hydrationFunction) == null ? void 0 : _f.call(options, data));
  res.json({
    data,
    total
  });
};

// src/getManyHandler.ts
var getManyHandler = async (req, res, table) => {
  const { ids } = req.body.params;
  const list = await table.findMany({
    where: { id: { in: ids } }
  });
  res.json({ data: list });
};

// src/getOneHandler.ts
var getOneHandler = async (req, res, table, arg) => {
  var _a, _b;
  const row = await table.findUnique({
    where: { id: +req.body.params.id },
    select: (_a = arg == null ? void 0 : arg.select) != null ? _a : void 0,
    include: (_b = arg == null ? void 0 : arg.include) != null ? _b : void 0
  });
  return res.json({ data: row });
};

// src/lib/isObject.ts
var isObject = (arg) => Object.prototype.toString.call(arg) === "[object Object]";

// src/updateHandler.ts
var updateHandler = async (req, res, table, options) => {
  const data = Object.entries(req.body.params.data).reduce((fields, [key, value]) => {
    var _a;
    if (!isObject(value) && !((_a = options == null ? void 0 : options.skipFields) == null ? void 0 : _a.includes(key)))
      fields[key] = value;
    return fields;
  }, {});
  const updated = await table.update({
    where: { id: +req.body.params.id },
    data
  });
  return res.json({ data: updated });
};

// src/deleteHandler.ts
var deleteHandler = async (req, res, table, options) => {
  const deleted = (options == null ? void 0 : options.softDeleteField) ? await table.update({
    where: { id: +req.body.params.id },
    data: {
      [options == null ? void 0 : options.softDeleteField]: new Date()
    }
  }) : await table.delete({
    where: { id: +req.body.params.id }
  });
  return res.json({ data: deleted });
};

// src/createHandler.ts
var createHandler = async (req, res, table) => {
  const created = await table.create({
    data: req.body.params.data
  });
  return res.json({ data: created });
};

// src/defaultHandler.ts
var defaultHandler = async (req, res, prisma, options) => {
  switch (req.body.method) {
    case "getList": {
      return await getListHandler(req, res, prisma[req.body.model || req.body.resource]);
    }
    case "getOne": {
      return await getOneHandler(req, res, prisma[req.body.model || req.body.resource]);
    }
    case "getMany": {
      return await getManyHandler(req, res, prisma[req.body.model || req.body.resource]);
    }
    case "getManyReference": {
      throw new Error("Not implemented yet");
    }
    case "create": {
      return await createHandler(req, res, prisma[req.body.model || req.body.resource]);
    }
    case "update": {
      return await updateHandler(req, res, prisma[req.body.model || req.body.resource]);
    }
    case "delete": {
      return await deleteHandler(req, res, prisma[req.body.model || req.body.resource], options);
    }
    case "deleteMany": {
      throw new Error("Not implemented yet");
    }
    default:
      throw new Error("Invalid method");
  }
};

// src/dataProvider.ts
var import_axios = __toESM(require("axios"));
var dataProvider = (endpoint, options) => {
  const apiService = import_axios.default.create({
    baseURL: endpoint
  });
  apiService.interceptors.response.use((res) => res.data);
  return {
    getList: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "getList",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    getOne: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "getOne",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    getMany: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "getMany",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    getManyReference: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "getManyReference",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    create: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "create",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    update: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "update",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    updateMany: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "updateMany",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    delete: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "delete",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    },
    deleteMany: (resource, params) => {
      var _a, _b;
      return apiService.post(resource, {
        method: "deleteMany",
        resource,
        params,
        model: (_b = (_a = options == null ? void 0 : options.resourceToModelMap) == null ? void 0 : _a[resource]) != null ? _b : void 0
      });
    }
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  dataProvider,
  defaultHandler,
  deleteHandler,
  getListHandler,
  getManyHandler,
  getOneHandler,
  updateHandler
});
